@@ -3,10 +3,8 @@
 
 #include <cmath> 
 #include <RcppEigen.h>
-#include "optim/optimmaths.h"
-#include "optim/matrixfield.h"
-#include "optim/optimclass.h"
-#include "optim/optimdata.h"
 #include <glmmr/openmpheader.h>
+#include "optim/optimdata.h"
+#include "optim/optimclass.h"
 
 #endif
