@@ -6,9 +6,12 @@
 #include "optimlinalg.h"
 #include "matrixfield.h"
 #include "eigenext.h"
-#include "optimmaths.h"
 #include "optimdata.h"
-#include <glmmr/openmpheader.h>
+#include "optimderivatives.h"
+#include <progress.hpp>
+#include <progress_bar.hpp>
+#include <glmmr.h>
+
 
 using namespace Eigen;
 
@@ -19,6 +22,7 @@ namespace glmmr {
 class OptimDesign {
 private:
   glmmr::OptimData& data_;
+  glmmr::OptimDerivatives& derivs_;
   ArrayXi curr_obs_;
   const int nlist_;
   int n_;
@@ -48,18 +52,23 @@ public:
   const bool robust_log_; // robust designs mode: true = log sum, false = sum
   bool trace_;
   bool uncorr_;
+  bool kr_;
   bool bayes_;
   
 public:
+  
   OptimDesign(const ArrayXi& idx_in, 
               int n,
               glmmr::OptimData& data,
+              glmmr::OptimDerivatives& deriv,
               int nmax,
               bool robust_log = false, 
               bool trace=false,
               bool uncorr=false,
+              bool kr = false,
               bool bayes = false) :
   data_(data),
+  derivs_(deriv),
   curr_obs_(data_.max_obs_.size()),//set to zero
   nlist_(data_.weights_.size()),
   n_(n), 
@@ -77,8 +86,10 @@ public:
   rm1A_list_(nmax_*nlist_,nmax_),//set to zero
   robust_log_(robust_log), 
   trace_(trace),
-  uncorr_(uncorr),
+  uncorr_(kr ? false : uncorr),
+  kr_(kr),
   bayes_(bayes){
+    if(bayes_ & kr_)Rcpp::stop("Can't have KR and Bayes");
     build_XZ();
   }
   
@@ -87,6 +98,8 @@ public:
   void local_search();
   void greedy_search();
   void reverse_greedy_search();
+  MatrixXd KR_correction(const MatrixXd& M,const int& idx,const MatrixXd& X,const MatrixXd& Z,const MatrixXd& Sinv,const ArrayXi& indexes);
+  double c_obj_fun(const MatrixXd &M,const VectorXd &C);
   
 private:
   ArrayXi get_rows(int idx);
@@ -109,6 +122,100 @@ inline ArrayXi glmmr::OptimDesign::join_idx(const ArrayXi &idx,
   return newidx;
 }
 
+inline MatrixXd glmmr::OptimDesign::KR_correction(const MatrixXd& M,
+                                              const int& idx,
+                                              const MatrixXd& X,
+                                              const MatrixXd& Z,
+                                              const MatrixXd& Sinv,
+                                              const ArrayXi& indexes){
+  if(idx>=derivs_.FirstOrderDerivatives.size() )Rcpp::stop("idx out of range");
+  int n = X.rows();
+  if(n != indexes.size())Rcpp::stop("indexes not equal to X array extent");
+  int R = derivs_.FirstOrderDerivatives[idx].size();
+  int Rmod = derivs_.gaussian[idx] == 1 ? R+1 : R;
+  MatrixXd M_new(M);
+  MatrixXd SigX = Sinv*X;
+  //MatrixXd PG = Sinv - SigX*M*SigX.transpose();
+  MatrixXd M_theta = MatrixXd::Zero(Rmod,Rmod);
+  MatrixXd partial1(n,n);
+  MatrixXd partial2(n,n);
+  MatrixXd meat = MatrixXd::Zero(X.cols(),X.cols());
+  glmmr::MatrixField<MatrixXd> P;
+  glmmr::MatrixField<MatrixXd> Q;
+  glmmr::MatrixField<MatrixXd> RR;
+  glmmr::MatrixField<MatrixXd> S;
+  int counter = 0;
+  for(int i = 0; i < Rmod; i++){
+    if(i < R){
+      partial1 = Z*derivs_.FirstOrderDerivatives[idx](i)*Z.transpose();
+    } else {
+      partial1 = MatrixXd::Identity(n,n);
+    }
+    P.add(-1*SigX.transpose()*partial1*SigX);
+    for(int j = i; j < Rmod; j++){
+      if(j < R){
+        partial2 = Z*derivs_.FirstOrderDerivatives[idx](j)*Z.transpose();
+      } else {
+        partial2 = MatrixXd::Identity(n,n);
+      }
+      S.add(Sinv*partial1*Sinv*partial2);
+      Q.add(X.transpose()*Sinv*partial1*Sinv*partial2*SigX);
+      // counter++;
+      if(i < R && j < R){
+        int scnd_idx = i + j*(R-1) - j*(j-1)/2;
+        if(scnd_idx >= derivs_.SecondOrderDerivatives[idx].size())Rcpp::stop("scnd > SoD s");
+        RR.add(SigX.transpose()*Z*derivs_.SecondOrderDerivatives[idx](scnd_idx)*Z.transpose()*SigX);
+      }
+    }
+  }
+  counter = 0;
+  for(int i = 0; i < Rmod; i++){
+    for(int j = i; j < Rmod; j++){
+      // using the expected REML information matrix here, but can use alternative commented out
+      M_theta(i,j) = 0.5*(S(counter).trace())- (M*Q(counter)).trace() + 0.5*((M*P(i)*M*P(j)).trace());
+      if(i!=j)M_theta(j,i)=M_theta(i,j);
+      counter++;
+    }
+  }
+  M_theta = M_theta.llt().solve(MatrixXd::Identity(Rmod,Rmod));
+  for(int i = 0; i < (Rmod-1); i++){
+    if(i < R){
+      partial1 = Z*derivs_.FirstOrderDerivatives[idx](i)*Z.transpose();
+    } else {
+      partial1 = MatrixXd::Identity(n,n);
+    }
+    for(int j = (i+1); j < Rmod; j++){
+      if(j < R){
+        partial2 = Z*derivs_.FirstOrderDerivatives[idx](j)*Z.transpose();
+      } else {
+        partial2 = MatrixXd::Identity(n,n);
+      }
+      int scnd_idx = i + j*(Rmod-1) - j*(j-1)/2;
+      meat += M_theta(i,j)*(Q(scnd_idx) + Q(scnd_idx).transpose() - P(i)*M*P(j) -P(j)*M*P(i));
+      //meat += M_theta(i,j)*(SigX.transpose()*partial2*PG*partial1*SigX);
+      if(i < R && j < R){
+        scnd_idx = i + j*(R-1) - j*(j-1)/2;
+        meat -= 0.5*M_theta(i,j)*(RR(scnd_idx));
+      }
+    }
+  }
+  for(int i = 0; i < Rmod; i++){
+    if(i < R){
+      partial1 = Z*derivs_.FirstOrderDerivatives[idx](i)*Z.transpose();
+    } else {
+      partial1 = MatrixXd::Identity(n,n);
+    }
+    int scnd_idx = i + i*(Rmod-1) - i*(i-1)/2;
+    meat += M_theta(i,i)*(Q(scnd_idx) - P(i)*M*P(i));
+    if(i < R){
+      scnd_idx = i + i*(R-1) - i*(i-1)/2;
+      meat -= 0.25*M_theta(i,i)*RR(scnd_idx);
+    }
+  }
+  M_new = M + 2*M*meat*M;
+  return M_new;
+}
+
 inline void glmmr::OptimDesign::build_XZ(){
   curr_obs_ = ArrayXi::Zero(curr_obs_.size());
   p_ = ArrayXi::Zero(nlist_);
@@ -133,13 +240,15 @@ inline void glmmr::OptimDesign::build_XZ(){
     MatrixXd Z = MatrixXd::Zero(nmax_,q_(j));
     VectorXd w_diag(nmax_);
     rowcount = 0;
+    std::vector<int> starting_rows;
     for(int k=0; k<idx_in_.size();k++){
-      ArrayXi rowstoincl = glmmr::Eigen_ext::find(data_.exp_cond_, idx_in_(k));
+      ArrayXi rowstoincl = glmmr::OptimEigen::find(data_.exp_cond_, idx_in_(k));
       for(int l=0;l<rowstoincl.size();l++){
         X.row(rowcount) = data_.X_all_list_.get_row(j,rowstoincl(l));
         Z.row(rowcount) = data_.Z_all_list_.get_row(j,rowstoincl(l));
         w_diag(rowcount) = data_.W_all_diag_(rowstoincl(l),j);
         if(j==0)count_exp_cond_(k)++;
+        starting_rows.push_back(rowstoincl(l));
         rowcount++;
       }
     }
@@ -148,16 +257,21 @@ inline void glmmr::OptimDesign::build_XZ(){
     MatrixXd tmp = Z.topRows(rowcount)*Dt*Z.topRows(rowcount).transpose();
     MatrixXd I = MatrixXd::Identity(tmp.rows(),tmp.cols());
     tmp.diagonal() += w_diag.head(rowcount);
-    MatrixXd M = X.topRows(rowcount).transpose() * tmp.llt().solve(I) * X.topRows(rowcount);
+    tmp = tmp.llt().solve(I);
+    MatrixXd M = X.topRows(rowcount).transpose() * tmp * X.topRows(rowcount);
+    if(bayes_){
+      M += data_.V0_list_(j);
+    }
+    M = M.llt().solve(MatrixXd::Identity(M.rows(),M.cols()));
+    if(kr_){
+      M = KR_correction(M,j,X.topRows(rowcount),Z.topRows(rowcount),tmp,Map<ArrayXi,Unaligned>(starting_rows.data(),starting_rows.size()));
+    }
     M_list_.add(M);
     M_list_sub_.add(M);
-    if(uncorr_){
-      vals(j) = bayes_ ? glmmr::maths::c_obj_fun( M_list_(j) + data_.V0_list_(j), data_.C_list_(j)) : glmmr::maths::c_obj_fun( M_list_(j), data_.C_list_(j));
-    } else {
-      A_list_.block(j*nmax_,0,r_in_design_,r_in_design_) = tmp.llt().solve(I);
-      vals(j) = bayes_ ? glmmr::maths::c_obj_fun( X.topRows(rowcount).transpose() * A_list_.block(j*nmax_,0,r_in_design_,r_in_design_) * X.topRows(rowcount) +
-        data_.V0_list_(j), data_.C_list_(j)) : glmmr::maths::c_obj_fun( X.topRows(rowcount).transpose() * A_list_.block(j*nmax_,0,r_in_design_,r_in_design_) * X.topRows(rowcount), data_.C_list_(j));
-    }
+    vals(j) = c_obj_fun( M, data_.C_list_(j));
+    if(!uncorr_){
+      A_list_.block(j*nmax_,0,r_in_design_,r_in_design_) = tmp;
+    } 
   }
   new_val_ = robust_log_ ? vals.log().transpose()*data_.weights_ : vals.transpose()*data_.weights_;
   if(trace_)Rcpp::Rcout << "\nStarting val: " << new_val_;
@@ -171,19 +285,22 @@ inline void glmmr::OptimDesign::local_search(){
   while(diff < 0){
     i++;
     val_ = new_val_;
-    if (trace_) Rcpp::Rcout << "\nIter " << i << ": Current value: " << val_;
+    if (trace_) Rcpp::Rcout << "\n|Iteration " << i << "| Current value: " << val_;
     // evaluate the swaps
     ArrayXXd val_swap = ArrayXXd::Constant(k_,k_,10000);
+    if( trace_) Rcpp::Rcout << "\nCalculating swaps: \n";
+    Progress bar(k_+1,trace_);
     for(int j=1; j < k_+1; j++){
       if((idx_in_ == j).any()){
         ArrayXd val_in_vec = eval(true,j); //eval is a member function of this class
         val_swap.row(j-1) = val_in_vec.transpose();
       }
+      bar.increment();
     }
     Index minrow, mincol;
     double newval = val_swap.minCoeff(&minrow,&mincol);
     diff = newval - val_;
-    if (trace_) Rcpp::Rcout << " || Best Difference: " << diff << " Best New value: " << newval;
+    if (trace_) Rcpp::Rcout << "\nBest Difference: " << diff << " | Best New value: " << newval;
     if(diff < 0){
       double tmp;
       if(uncorr_){
@@ -211,7 +328,7 @@ inline void glmmr::OptimDesign::greedy_search(){
     i++;
     idxcount++;
     val_ = new_val_;
-    if (trace_) Rcpp::Rcout << "\nIter " << i << " size: " << idxcount << " Current value: " << val_ ;
+    if (trace_) Rcpp::Rcout << "\n|Iteration " << i << "| Size: " << idxcount << " Current value: " << val_ ;
     ArrayXd val_swap = eval(false);
     Index swap_sort;
     double min = val_swap.minCoeff(&swap_sort);
@@ -236,7 +353,9 @@ inline void glmmr::OptimDesign::reverse_greedy_search(){
   while(idxcount > n_){
     i++;
     val_ = new_val_;
-    if (trace_) Rcpp::Rcout << "\nIter " << i << " size: " << idxcount << " Current value: " << val_ ;
+    if (trace_) Rcpp::Rcout << "\n|Iteration " << i << "| Size: " << idxcount << " Current value: " << val_ ;
+    if( trace_) Rcpp::Rcout << "\nCalculating removals: \n";
+    Progress bar(k_+1,trace_);
     for(int j = 1; j< k_+1; j++){
       if((idx_in_ == j).any()){
         if(uncorr_){
@@ -247,11 +366,12 @@ inline void glmmr::OptimDesign::reverse_greedy_search(){
       } else {
         val_rm(j-1) = 10000;
       }
+      bar.increment();
     }
     Index swap_sort;
     double min = val_rm.minCoeff(&swap_sort);
     (void)min;
-    if (trace_) Rcpp::Rcout << " removing " << swap_sort+1;
+    if (trace_) Rcpp::Rcout << "\nRemoving: " << swap_sort+1;
     if(uncorr_){
       new_val_ = rm_obs_uncor((int)swap_sort+1,true,true,true);
     } else {
@@ -274,7 +394,7 @@ inline ArrayXi glmmr::OptimDesign::get_all_rows(ArrayXi idx){
   ArrayXi rowidx(nmax_);
   int count = 0;
   for(int i = 0; i < idx.size(); i++){
-    ArrayXi addidx = glmmr::Eigen_ext::find(data_.exp_cond_,idx(i));
+    ArrayXi addidx = glmmr::OptimEigen::find(data_.exp_cond_,idx(i));
     rowidx.segment(count,addidx.size()) = addidx;
     count += addidx.size();
   }
@@ -297,7 +417,7 @@ inline double glmmr::OptimDesign::rm_obs(int outobs,
               bool keep,
               bool keep_mat,
               bool rtn_val){
-  ArrayXi rm_cond = glmmr::Eigen_ext::find(idx_in_,outobs);
+  ArrayXi rm_cond = glmmr::OptimEigen::find(idx_in_,outobs);
   ArrayXi rowstorm = get_rows(rm_cond(0));
   idx_in_rm_ = glmmr::algo::uvec_minus(idx_in_,rm_cond(0));
   ArrayXi idxexist = get_all_rows(idx_in_rm_);
@@ -309,10 +429,23 @@ inline double glmmr::OptimDesign::rm_obs(int outobs,
     if(idx==0)r_in_rm_ = rm1A.rows();
     rm1A_list_.block(idx*nmax_,0,r_in_rm_,r_in_rm_) = rm1A;
     int p = data_.X_all_list_.cols(idx);
-    MatrixXd X = glmmr::Eigen_ext::mat_indexing(data_.X_all_list_(idx),idxexist,ArrayXi::LinSpaced(p,0,p-1));
+    MatrixXd X = glmmr::OptimEigen::mat_indexing(data_.X_all_list_(idx),idxexist,ArrayXi::LinSpaced(p,0,p-1));
     MatrixXd M = X.transpose()*rm1A*X;
-    M_list_sub_.replace(idx,M);
-    if(rtn_val)vals(idx) = bayes_ ? glmmr::maths::c_obj_fun( M+data_.V0_list_(idx), data_.C_list_(idx)) : glmmr::maths::c_obj_fun( M, data_.C_list_(idx));
+    if(bayes_){
+      M += data_.V0_list_(idx);
+    }
+    bool issympd = glmmr::OptimEigen::isnotsympd(M);
+    if(!issympd){
+      M = M.llt().solve(MatrixXd::Identity(M.rows(),M.cols()));
+      if(kr_){
+        int q = data_.Z_all_list_.cols(idx);
+        MatrixXd Z = glmmr::OptimEigen::mat_indexing(data_.Z_all_list_(idx),idxexist,ArrayXi::LinSpaced(q,0,q-1));
+        MatrixXd Mkr(M);
+        M = KR_correction(Mkr,idx,X,Z,rm1A,idxexist);
+      } 
+      M_list_sub_.replace(idx,M);
+      if(rtn_val)vals(idx) = c_obj_fun( M, data_.C_list_(idx));
+    }
     if(keep_mat){
       M_list_.replace(idx,M);
       A_list_.block(idx*nmax_,0,r_in_rm_,r_in_rm_) = rm1A;
@@ -340,7 +473,7 @@ inline double glmmr::OptimDesign::rm_obs_uncor(int outobs,
                     bool keep,
                     bool keep_mat,
                     bool rtn_val){
-  ArrayXi rm_cond = glmmr::Eigen_ext::find(idx_in_,outobs);
+  ArrayXi rm_cond = glmmr::OptimEigen::find(idx_in_,outobs);
   ArrayXi rowstorm = get_rows(rm_cond(0));
   VectorXd vals = VectorXd::Constant(nlist_,10000.0); 
   
@@ -362,9 +495,7 @@ inline double glmmr::OptimDesign::rm_obs_uncor(int outobs,
     tmp = tmp.llt().solve(I);
     M.noalias() -= X.transpose()*tmp*X;
     M_list_sub_.replace(j,M);
-    if(rtn_val)vals(j) = bayes_ ? glmmr::maths::c_obj_fun( M+data_.V0_list_(j), data_.C_list_(j)) : glmmr::maths::c_obj_fun( M, data_.C_list_(j));
-    
-    
+    if(rtn_val)vals(j) = bayes_ ? c_obj_fun( M+data_.V0_list_(j), data_.C_list_(j)) : c_obj_fun( M, data_.C_list_(j));
     if(keep_mat){
       M_list_.replace(j,M);
     }
@@ -393,7 +524,7 @@ inline double glmmr::OptimDesign::rm_obs_uncor(int outobs,
 inline double glmmr::OptimDesign::add_obs(int inobs,
                bool userm,
                bool keep ){
-  ArrayXi rowstoadd = glmmr::Eigen_ext::find(data_.exp_cond_,inobs);
+  ArrayXi rowstoadd = glmmr::OptimEigen::find(data_.exp_cond_,inobs);
   ArrayXi idxvec = userm ? idx_in_rm_ : idx_in_;
   ArrayXi idxexist = get_all_rows(idxvec);
   int n_to_add = rowstoadd.size();
@@ -402,41 +533,55 @@ inline double glmmr::OptimDesign::add_obs(int inobs,
   VectorXd vals(nlist_);
   bool issympd;
   int r_in_design_tmp_ = r_in_design_;
+  bool exit_loop = false;
   for (int idx = 0; idx < nlist_; ++idx) {
     MatrixXd A = userm ? rm1A_list_.block(idx*nmax_,0,r_in_rm_,r_in_rm_) : 
     A_list_.block(idx*nmax_,0,r_in_design_tmp_,r_in_design_tmp_);
     n_already_in = idxexist.size();
     MatrixXd X = MatrixXd::Zero(n_already_in + n_to_add,p_(idx));
+    MatrixXd Z = MatrixXd::Zero(n_already_in + n_to_add,q_(idx));
     idx_in_vec = ArrayXi::Zero(n_already_in + n_to_add);
     idx_in_vec.segment(0,n_already_in) = idxexist;
     MatrixXd X0 = data_.X_all_list_(idx);
-    X.block(0,0,n_already_in,X.cols()) = glmmr::Eigen_ext::mat_indexing(X0,idxexist,ArrayXi::LinSpaced(X0.cols(),0,X0.cols()-1));
+    MatrixXd Z0 = data_.Z_all_list_(idx);
+    X.block(0,0,n_already_in,X.cols()) = glmmr::OptimEigen::mat_indexing(X0,idxexist,ArrayXi::LinSpaced(X0.cols(),0,X0.cols()-1));
+    Z.block(0,0,n_already_in,Z.cols()) = glmmr::OptimEigen::mat_indexing(Z0,idxexist,ArrayXi::LinSpaced(Z0.cols(),0,Z0.cols()-1));
     MatrixXd D = data_.D_list_(idx);
     for(int j = 0; j < n_to_add; j++){
       RowVectorXd z_j = data_.Z_all_list_.get_row(idx,rowstoadd(j));
       int zcols = data_.Z_all_list_.cols(idx);
-      MatrixXd z_d = glmmr::Eigen_ext::mat_indexing(data_.Z_all_list_(idx),idx_in_vec.segment(0,n_already_in),ArrayXi::LinSpaced(zcols,0,zcols-1));
+      MatrixXd z_d = glmmr::OptimEigen::mat_indexing(data_.Z_all_list_(idx),idx_in_vec.segment(0,n_already_in),ArrayXi::LinSpaced(zcols,0,zcols-1));
       double sig_jj = z_j * D * z_j.transpose(); 
       sig_jj += data_.W_all_diag_(rowstoadd(j),idx);
       VectorXd f = z_d * D * z_j.transpose();
       A = glmmr::algo::add_one_mat(A, sig_jj,f);
       idx_in_vec(n_already_in) = rowstoadd(j);
       X.row(n_already_in) = data_.X_all_list_.get_row(idx,rowstoadd(j));  
+      Z.row(n_already_in) = data_.Z_all_list_.get_row(idx,rowstoadd(j));  
       n_already_in++;
     }
     MatrixXd M = X.transpose() * A * X;
-    issympd = glmmr::Eigen_ext::issympd(M);
+    if(bayes_){
+      M += data_.V0_list_(idx);
+    }
+    issympd = glmmr::OptimEigen::isnotsympd(M);
     if(!issympd){
+      M = M.llt().solve(MatrixXd::Identity(M.rows(),M.cols()));
+      if(kr_){
+        MatrixXd Mkr(M);
+        M = KR_correction(Mkr,idx,X,Z,A,idx_in_vec);
+      }
       if(keep){
         if(idx==0)r_in_design_ = A.rows();
         M_list_.replace(idx,M);
         A_list_.block(idx*nmax_,0,r_in_design_,r_in_design_) = A;
       }
-      vals(idx) = bayes_ ? glmmr::maths::c_obj_fun( M+data_.V0_list_(idx), data_.C_list_(idx)) : glmmr::maths::c_obj_fun( M, data_.C_list_(idx));
+      vals(idx) = c_obj_fun( M, data_.C_list_(idx));
     } else {
       for(int k = 0; k<vals.size(); k++)vals(k) = 10000;
-      break;
+      exit_loop = true;
     }
+    if(exit_loop)break;
   }
   if(keep && !issympd){
     if(userm){
@@ -462,8 +607,9 @@ inline double glmmr::OptimDesign::add_obs_uncor(int inobs,
                      bool userm,
                      bool keep){
   VectorXd vals(nlist_);
-  ArrayXi rowstoadd = glmmr::Eigen_ext::find(data_.exp_cond_,inobs);
+  ArrayXi rowstoadd = glmmr::OptimEigen::find(data_.exp_cond_,inobs);
   bool issympd = true;
+  bool exit_loop = false;
   for(int j=0; j<nlist_;j++){
     MatrixXd X = MatrixXd::Zero(rowstoadd.size(),p_(j));
     MatrixXd Z = MatrixXd::Zero(rowstoadd.size(),q_(j));
@@ -479,16 +625,17 @@ inline double glmmr::OptimDesign::add_obs_uncor(int inobs,
     MatrixXd M = userm ? M_list_sub_(j) : M_list_(j);
     MatrixXd iden = MatrixXd::Identity(tmp.rows(),tmp.cols());
     M += X.transpose() * tmp.llt().solve(iden) * X;
-    issympd = glmmr::Eigen_ext::issympd(M);
+    issympd = glmmr::OptimEigen::isnotsympd(M);
     if(!issympd){
       if(keep){
         M_list_.replace(j,M);
       }
-      vals(j) = bayes_ ? glmmr::maths::c_obj_fun( M+data_.V0_list_(j), data_.C_list_(j)) : glmmr::maths::c_obj_fun( M, data_.C_list_(j));
+      vals(j) = bayes_ ? c_obj_fun( M+data_.V0_list_(j), data_.C_list_(j)) : c_obj_fun( M, data_.C_list_(j));
     } else {
       for(int k = 0; k<vals.size(); k++)vals(k) = 10000;
-      break;
+      exit_loop = true;
     }
+    if(exit_loop)break;
   }
   if(keep && !issympd){
     if(userm){
@@ -550,4 +697,9 @@ inline ArrayXd glmmr::OptimDesign::eval(bool userm, int obs){
   return val_in_mat;
 }
 
+inline double glmmr::OptimDesign::c_obj_fun(const MatrixXd &M, 
+                        const VectorXd &C) {
+  return C.transpose() * M * C;
+}
+
 #endif
